/*
 * File:   main_vlcd.c
 * Author: Steffen Hofmann
 *
 * Created on 18. November 2024, 16:47
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include "hw.h"
#include <util/delay.h>
#include "ports.h"
#include "vlcd_spi.h"
#include "TWI0_host.h"
#include "aht20.h"
#include "vlcd_pwm.h"
#include "ugui.h"
#include "ugui_S1D15721.h"


#define BuffOUT     40

char bufferOut[BuffOUT];
char bufferTemp[BuffOUT];
char bufferHumi[BuffOUT];
FUSES =
{
.BODCFG = ACTIVE_DISABLE_gc | LVL_BODLEVEL0_gc | SAMPFREQ_128Hz_gc | SLEEP_DISABLE_gc,
.BOOTSIZE = 0x0,
.CODESIZE = 0x0,
.OSCCFG = CLKSEL_OSCHF_gc,
.SYSCFG0 = CRCSEL_CRC16_gc | CRCSRC_NOCRC_gc | RSTPINCFG_GPIO_gc | UPDIPINCFG_UPDI_gc,
.SYSCFG1 = MVSYSCFG_DUAL_gc | SUT_0MS_gc,
.WDTCFG = PERIOD_OFF_gc | WINDOW_OFF_gc,
};





int main(void) {
   
   teErrorNumber TeErrorNumber = NO_ERROR;

   _PROTECTED_WRITE (CLKCTRL.OSCHFCTRLA, (CLKCTRL_FRQSEL_24M_gc | CLKCTRL_AUTOTUNE_bm));
   
    init_ports();
    SPI0_init();
    TWI0_initHost();  //- 100KHz, PullUp Enable
    TWI0_initPins();
    pwm_init();  //- 1KHz
    
    RST_LCD_LOW();
    _delay_ms(5);
    RST_LCD_HIGH();
    


    Display_Init();

    UG_SetBackcolor(C_BLACK);
    UG_SetForecolor(C_WHITE);  
    

    /* Clear screen. */
    UG_FillScreen(C_BLACK);
    
    UG_DrawFrame(0, 0, DISPLAY_WIDTH-1, DISPLAY_HEIGHT-1, C_WHITE);     // Draw a Frame    
   
   // UG_DrawCircle( 100, 20, 20, C_WHITE );
   
    //Some Texts
    UG_FontSelect(&FONT_10X16);
    UG_SetBackcolor(C_BLACK);
#ifdef GRAY_SCALE
    UG_SetForecolor(C_LIGHT_GRAY);
#else    
    UG_SetForecolor(C_WHITE);  
#endif
    sprintf(bufferOut,"S1D15721 GUI %dx%d", DISPLAY_WIDTH, DISPLAY_HEIGHT);
    UG_PutString(10, 10 , bufferOut);
    
    UG_FontSelect(&FONT_8X8);
    
    //main_i2c_scan(I2C_SCAN_MIN, I2C_SCAN_MAX);
    TeErrorNumber = intAHT20();
    
    
    /* Replace with your application code */
    while (1) {
     
      if( TeErrorNumber == NO_ERROR )
      {
 	 	if( AHT20_available() == true )
	 	 {
	 	   		 		
	 	   	float temperature = AHT20_getTemperature();
            float humidity = AHT20_getHumidity();
            
            sprintf(bufferHumi,"Humity AHT25: %.2f \%", humidity);
            
            if( temperature > 0 ){
               sprintf(bufferTemp,"Temp AHT25: +%.2f \xf8\C", temperature);
            }
            else {
               sprintf(bufferTemp,"Temp AHT25: -%.2f \xf8\C", temperature); 
            }
              
	 	    UG_PutString(5, 30 , bufferTemp);	
            UG_PutString(5, 40 , bufferHumi); 
            
            
         }

      }
      else
      {
            sprintf(bufferOut,"AHT20 not available !");
            UG_PutString(5, 50 , bufferOut);
      }
         _delay_ms(3000); //- 3 sec.
    }
}
