/*******************************************************************************
*                       (c)2024 shotech Steffen Hofmann                        *
*                           D-22525 Hamburg                                    *
********************************************************************************
*  $Source: /home/CVS/,v $
*------------------------------------------------------------------------------*
*  Datum der 1. Version  :  15.12.2024
*  Geraete Bezeichnung   :  VLCD
*------------------------------------------------------------------------------*
*- $Id:  $
*------------------------------------------------------------------------------*
*
*	Headerfile
*	----------
*	ugui_S1D15721 driver SPI
*
******************************************************************************/

#include "ugui.h"
#include "ugui_S1D15721.h"
#include "vlcd_spi.h"
#include "hw.h"
#include "types.h"


#define HDP (DISPLAY_WIDTH - 1)
#define VDP (DISPLAY_HEIGHT - 1)


#define CS_HIGH() VLCD_CS_HIGH()
#define CS_LOW()  VLCD_CS_LOW()

UG_U16 page_offset;
UG_U16 col_offset;
//const UG_U16 screensize_X = DISPLAY_WIDTH;
//const UG_U16 screensize_Y = DISPLAY_HEIGHT;
//const UG_U16 _LCDPAGES = DISPLAY_PAGES;
#ifdef GRAY_SCALE
unsigned char LCD_buffer[DIMENSION_X4];
#else
unsigned char LCD_buffer[DIMENSION_X8];
#endif

/* *** Glogal GUI structure for this. *** */
UG_GUI guiS1D15721;
//- Local
void Display_PSet(UG_S16 x, UG_S16 y, UG_COLOR c);

//-----------------------------------------------------------------------------
// Sends Data (1 Byte).
//-----------------------------------------------------------------------------
static void _lcd_data(uint8_t data) {
	CS_LOW();
	A0_HIGH();
	SPI0_exchangeData(data);
	CS_HIGH();
}
//-----------------------------------------------------------------------------
// Sends Commands (1 Byte).
//-----------------------------------------------------------------------------
static void _lcd_cmd1(uint8_t cmd) {
	CS_LOW();
	A0_LOW();
	SPI0_exchangeData(cmd);
	CS_HIGH();
}
//-----------------------------------------------------------------------------
// Sends Commands (2 Byte).
//-----------------------------------------------------------------------------
static void _lcd_cmd2(uint8_t cmd, uint8_t arg) {
	CS_LOW();
	A0_LOW();
	SPI0_exchangeData(cmd);
	A0_HIGH();
	SPI0_exchangeData(arg);
	CS_HIGH();
}
//-----------------------------------------------------------------------------
// Sends Commands (3 Byte).
//-----------------------------------------------------------------------------
static void _lcd_cmd3(uint8_t cmd, uint8_t arg1, uint8_t arg2) {
	CS_LOW();
	A0_LOW();
	SPI0_exchangeData(cmd);
	A0_HIGH();
	SPI0_exchangeData(arg1);
	SPI0_exchangeData(arg2);
	CS_HIGH();
}
//-----------------------------------------------------------------------------
//- init Dispay
//-----------------------------------------------------------------------------
void Display_Init()
{
    col_offset = 8;
    page_offset = 1;
    
    
     // Software Reset
    _lcd_cmd1(LCD_CMD_RESET);
//    usleep(1000);
    
    // Init Sequence
    _lcd_cmd2(LCD_CMD_BUILTIN_OSC_FREQ,0x00);
    _lcd_cmd1(LCD_CMD_BUILTIN_OSC|1);
    _lcd_cmd2(LCD_CMD_LC_DRIVE_VOLTAGE,0x07);
    _lcd_cmd2(LCD_CMD_EL_VOLUME,0x10);
    _lcd_cmd2(LCD_CMD_PWR_CONTROL,0x1F); //0b11111);
    _lcd_cmd3(LCD_CMD_DUTY_SET,0x11,0x00);

    _lcd_cmd1(LCD_CMD_DISPLAY_REVERSE|0);
    _lcd_cmd1(LCD_CMD_DISPLAY_ALL_LIGHT|0);
   
#ifdef GRAY_SCALE
     _lcd_cmd2(LCD_CMD_DISPLAY_MODE,0x00); //0b00);
#else
     _lcd_cmd2(LCD_CMD_DISPLAY_MODE,0x01); //0b00); keine gray scale
#endif
    _lcd_cmd2(LCD_CMD_GRAY_SCALE_PATTERN,0x36);
    _lcd_cmd1(LCD_CMD_COM_OUT_STATUS|1);
    _lcd_cmd2(LCD_CMD_DISPLAY_START_LINE,0);
    _lcd_cmd1(LCD_CMD_DISPLAY_OFFON|1);

#ifdef GRAY_SCALE
    memset( LCD_buffer, 0x00, DIMENSION_X4);
#else    
    memset( LCD_buffer, 0x00, DIMENSION_X8);
#endif
    // Initialize global structure and set PSET to this.PSET. 
    UG_Init(&guiS1D15721, Display_PSet, DISPLAY_WIDTH, DISPLAY_HEIGHT); 

}
//-----------------------------------------------------------------------------
//- callb. Pixel Set funktion ( 4 x Color )
//
// RAM data (top to bottom; left to right)
// Pixel(x,y):          index = y/8 + x*8
//                            = (y >> 3) + (x << 3)
//                      bit   = y%8
//                            = (y & 0x07)
// Set pixel (x,y):     disp_ram[(y >> 3) + (x << 3)] |=  (1 << (y & 0x07));
// Clear pixel (x,y):   disp_ram[(y >> 3) + (x << 3)] &= ~(1 << (y & 0x07));
//-----------------------------------------------------------------------------
#ifdef GRAY_SCALE
void Display_PSet(UG_S16 x, UG_S16 y, UG_COLOR color )
{    

    volatile u8 bPixel;
    volatile u16 uDimension;
    
 
	if (x < DISPLAY_WIDTH && y < DISPLAY_HEIGHT) {
		

        _lcd_cmd2(LCD_CMD_COL_ADDR, x+col_offset);
    
        _lcd_cmd2(LCD_CMD_PAGE_ADDR, ((y >> 2)+page_offset)&0x1F);
    
    
        _lcd_cmd1(LCD_CMD_DATA_WRITE);


        uDimension = (y >> 2) + (x << 2);
        bPixel = LCD_buffer[uDimension];
        
        if (color == C_WHITE )
        {
            
            bPixel &= ~(1 << (y & 0x07));
            bPixel &= ~(2 << (y & 0x07));
            LCD_buffer[uDimension] = bPixel;
        }
			
		   else if( color == C_LIGHT_GRAY )
        {
            bPixel |=  (1 << (y & 0x07));
            bPixel &= ~(2 << (y & 0x07));
            LCD_buffer[uDimension] = bPixel;
        
        }
		   else if( color == C_DARK_GRAY )
        {
            bPixel &= ~(1 << (y & 0x07));
            bPixel |=  (2 << (y & 0x07));
            LCD_buffer[uDimension] = bPixel;
        
        }
		   else if( color == C_BLACK )
        {
            bPixel |=  (1 << (y & 0x07));
            bPixel |=  (2 << (y & 0x07));
            LCD_buffer[uDimension] = bPixel;
        
        }
        else
        	{
        		
        	}
        			
        _lcd_data(bPixel);
    }    
}
#else
void Display_PSet(UG_S16 x, UG_S16 y, UG_COLOR color )
{    

    volatile u8 bPixel;
    volatile u16 uDimension;
    
 
	if (x < DISPLAY_WIDTH && y < DISPLAY_HEIGHT) {
		

        _lcd_cmd2(LCD_CMD_COL_ADDR, x+col_offset);
    
        _lcd_cmd2(LCD_CMD_PAGE_ADDR, ((y >> 3)+page_offset)&0x1F);
    
    
        _lcd_cmd1(LCD_CMD_DATA_WRITE);


        uDimension = (y >> 3) + (x << 3);
        bPixel = LCD_buffer[uDimension];
        
        if (color != C_BLACK )
        {
            
            bPixel |=  (1 << (y & 0x07));
            LCD_buffer[uDimension] = bPixel;
        }
			
		else
        {
            bPixel &= ~(1 << (y & 0x07));
            LCD_buffer[uDimension] = bPixel;
        
        }
		
        _lcd_data(bPixel);
    }    
}
#endif
