/******************************************************************************
*                              S.Hofmann                                      *
*                            info@shotech.de                                  *
*******************************************************************************
*  $Source: $
*-----------------------------------------------------------------------------*
*  Datum der 1. Version  :  27.05.2024 19:58:59
*  Geraete Bezeichnung   :  vlcd adapter
*-----------------------------------------------------------------------------*
*- $Id: $
*-----------------------------------------------------------------------------*
*
*	Sourcefile
*	----------
*	pwm
*
******************************************************************************/


#include "hw.h"
#include "vlcd_pwm.h"

static unsigned int VLCD_Period   = 0xFFFF;

void takeOverTCA0(void);
//- long map(value, long in_min, long in_max, long out_min, long out_max);
long map(long x, long in_min, long in_max, long out_min, long out_max);

void setDutyCycle(u8 duty );

void setFrequency(unsigned long freqInHz );



//-----------------------------------------------------------------------------
//- Reset TCA0
//-----------------------------------------------------------------------------
void takeOverTCA0() {
  TCA0.SPLIT.CTRLA = 0;               // Stop TCA0
  TCA0.SPLIT.CTRLESET = TCA_SPLIT_CMD_RESET_gc | 0x03; // Reset TCA0
}
//-----------------------------------------------------------------------------
//- 
//-----------------------------------------------------------------------------
long map(long x, long in_min, long in_max, long out_min, long out_max)
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}
//-----------------------------------------------------------------------------
//- Set duty
//-----------------------------------------------------------------------------
void setDutyCycle(u8 duty ) {
  TCA0.SINGLE.CMP1 = map(duty, 0, 255, 0, VLCD_Period);
}
//-----------------------------------------------------------------------------
//- Set PWM-Frequenz
//-----------------------------------------------------------------------------
void setFrequency(unsigned long freqInHz) {
  unsigned long tempperiod = (F_CPU / freqInHz);
  
  u8  presc = 0;
  while (tempperiod > 65536 && presc < 7) {
    presc++;
    tempperiod = tempperiod >> (presc > 4 ? 2 : 1);
  }
  VLCD_Period = tempperiod;
  TCA0.SINGLE.CTRLA = (presc << 1) | TCA_SINGLE_ENABLE_bm;
  TCA0.SINGLE.PER = VLCD_Period;
}

//-----------------------------------------------------------------------------
//- Set PWM and Duty
//-----------------------------------------------------------------------------
void PWM_Vlcd(unsigned long frequency, u8 duty){
  setFrequency(frequency);
  //setDutyCycle(64,Period); // ~25%
  //_delay(4000);
  //setDutyCycle(128, Period);// ~50%
  //_delay(4000);
  //setDutyCycle(192, Period);// ~75%
 // delay(4000);
  
  setDutyCycle(duty);
}

void pwm_init( void )
{
    
PORTMUX.TCAROUTEA = PORTMUX_TCA0_PORTD_gc;
 takeOverTCA0();                             // This replaces disabling and resettng the timer, required previously.
 //-  TCA_SINGLE_CMP1EN_bm = PD1 ( W1)
 TCA0.SINGLE.CTRLB   = (TCA_SINGLE_CMP1EN_bm | TCA_SINGLE_WGMODE_SINGLESLOPE_gc);
                                              // Single slope PWM mode, PWM on WO0
  TCA0.SINGLE.PER     = VLCD_Period;               // Count all the way up to 0xFFFF
  TCA0.SINGLE.CMP1    = 0;                    // At 20MHz, this gives ~305Hz PWM
  TCA0.SINGLE.CTRLA   = TCA_SINGLE_ENABLE_bm; // Enable the timer with no prescaler
  //- 50%  PWM
  PWM_Vlcd( PWM_1KHZ, BACKLIGHT_DUTY(50)); //- 1KHz, 50% duty ( 128 ))


  BL_HIGH();  // Backl. ON
    
}